#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
#  peng.py
#  
#  Copyright 2016 notna <notna@apparat.org>
#  
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#  
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#  
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
#  MA 02110-1301, USA.
#  
#  

__all__ = ["Peng"]

import pyglet

from . import window, config


class Peng(object):
    """
    This Class should only be instatiated once per application, if you want to use multiple windows, see :py:meth:`createWindow()`\ .
    
    An Instance of this class represents the whole Engine, with all accompanying state and window/world objects.
    
    Be sure to keep your instance accessible, as it will be needed to create most other classes.
    """
    
    def __init__(self):
        self.window = None
        
        self.cfg = config.Config({},defaults=config.DEFAULT_CONFIG)
    
    def createWindow(self,cls=window.PengWindow,*args,**kwargs):
        """
        createWindow(cls=window.PengWindow, *args, **kwargs)
        
        Creates a new window using the supplied ``cls``\ .
        
        If ``cls`` is not given, :py:class:`peng3d.window.PengWindow()` will be used.
        
        Any other positional or keyword arguments are passed to the class constructor.
        
        Note that this method currently does not support using multiple windows.
        
        .. todo::
           
           Implement having multiple windows.
        """
        self.window = cls(self,*args,**kwargs)
    
    def run(self):
        """
        Runs the application main loop.
        
        This method is blocking and needs to be called from the main thread to avoid OpenGL bugs that can occur.
        """
        self.window.run()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
#  version.py
#  
#  Copyright 2016 notna <notna@apparat.org>
#  
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#  
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#  
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
#  MA 02110-1301, USA.
#  
#  

__all__ = ["VERSION","RELEASE"]

VERSION = "1.1.0a1"
"""
Full version number of format ``MAJOR.MINOR.BUGFIX(a|b|pre)SUBRELEASE`` where major is increased only on very major feature changes.
Minor is changed if a new feature is introduced or an API change is made, while bugfix only changes if an important bugfix needs to be provided before the next release.
Either a, b, or pre follows depending on the type of release, e.g. a for alpha, b for beta and pre for prereleases.

Used to display the version in the title of the documentation.

.. seealso::
   
   See the `Distutils docs on version numbers <https://docs.python.org/2/distutils/setupscript.html#additional-meta-data>`_ for more information.

"""

RELEASE = "1.1.0"
"""
Same as :py:data:`VERSION` but without the subrelease.

Used to display the version on the top-right of the documentation.
"""



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
#  menu.py
#  
#  Copyright 2016 notna <notna@apparat.org>
#  
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#  
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#  
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
#  MA 02110-1301, USA.
#  
#  

from .layer import Layer

class BasicMenu(object):
    """
    Menu base class without layer support.
    
    Each menu is separated from the other menus and can be switched between at any time.
    
    .. seealso::
       
       See :py:class:`Menu()` for more information.
    
    """
    
    def __init__(self,name,window,peng):
        self.name = name
        self.window = window
        self.peng = peng
    def draw(self):
        """
        This method is called if it is time to render the menu.
        
        Override this method in subclasses to cutomize behaviour and actually draw stuff.
        """
        pass
    
    # Event handlers
    def on_enter(self,old):
        """
        This fake event handler will be called everytime this menu is entered via the :py:meth:`PengWindow.changeMenu()`\ .
        
        This handler will not be called if this menu is already active.
        """
        pass # Custom fake event handler for entering the menu

class Menu(BasicMenu):
    """
    Subclass of :py:class:`BasicMenu` adding support for the :py:class:`Layer` class.
    
    This subclass overrides the draw and __init__ method, so be sure to call them if you override them.
    """
    
    def __init__(self,name,window,peng):
        super(Menu,self).__init__(name,window,peng)
        self.layers = []
    def addLayer(self,layer,z=-1):
        """
        Adds a new layer to the stack, optionally at the specified z-value.
        
        ``layer`` must be an instance of Layer or subclasses.
        
        ``z`` can be used to override the index of the layer in the stack. Defaults to ``-1`` for appending.
        """
        # Adds a new layer to the stack, optionally at the specified z-value
        # The z-value is the index this layer should be inserted in, or -1 for appending
        if not isinstance(layer,Layer):
            raise TypeError("layer must be an instance of Layer!")
        if z==-1:
            self.layers.append(layer)
        else:
            self.layers.insert(z,layer)
    def draw(self):
        """
        Draws the layers in the appropriate order.
        
        Layers that have their ``enabled`` attribute set to ``False`` are skipped.
        """
        # Draws each layer in the given order
        for layer in self.layers:
            if layer.enabled:
                layer._draw()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
#  layer.py
#  
#  Copyright 2016 notna <notna@apparat.org>
#  
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#  
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#  
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
#  MA 02110-1301, USA.
#  
#  

__all__ = ["Layer", "Layer2D", "Layer3D"]

class Layer(object):
    """
    Base layer class.
    
    A Layer can be used to seperate background from foreground or the 3d world from a 2d HUD.
    
    This class by itself does nothing, you will need to subclass it and override the :py:meth:`draw()` method.
    """
    
    def __init__(self,menu,window,peng):
        self.window = window
        self.menu = menu
        self.peng = peng
        self.enabled = True
    
    # Subclass overrides
    def draw(self):
        """
        Called when this layer needs to be drawn.
        
        Override this method in subclasses to redefine behaviour.
        """
        pass
    def predraw(self):
        """
        Called before the :py:meth:`draw()` method is called.
        
        This method is used in the :py:class:`Layer2D()` and :py:class:`Layer3D()` subclasses for setting OpenGL state.
        
        Override this method in subclasses to redefine behaviour.
        """
        pass
    def postdraw(self):
        """
        Called after the :py:meth:`draw()` method is called.
        
        This method can be used to reset OpenGL state to avoid conflicts with other code.
        
        Override this method in subclasses to redefine behaviour.
        """
        pass
    # End subclass overrides
    
    def _draw(self):
        if not self.enabled:
            return
        self.predraw()
        try:
            self.draw()
        except Exception:
            raise
        finally:
            self.postdraw()

class Layer2D(Layer):
    """
    2D Variant of :py:class:`Layer()` and a subclass of the former.
    
    This class makes use of the :py:meth:`predraw()` method to configure OpenGL to draw 2-Dimensionally.
    This class uses :py:meth:`PengWindow.set2d()` to set the 2D mode.
    
    When overriding the :py:meth:`predraw()` method, make sure to call the superclass.
    """
    def predraw(self):
        self.window.set2d()

class Layer3D(Layer):
    """
    3D Variant of :py:class:`Layer()` and a subclass of the former.
    
    This class works the same as :py:class:`Layer2D()`\ , only for 3D drawing instead.
    This class uses :py:meth:`PengWindow.set3d()` to set the 3D mode.
    
    Also, the correct :py:func:`glTranslatef()` and :py:func:`glRotatef()` are applied to simplify drawing objects.
    When using the :py:meth:`draw()` method of this class, you will only need to use world coordinates, not camera coordinates.
    This allows for easy building of Games using First-Person-Perspectives.
    """
    def predraw(self):
        self.window.set3d()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
#  window.py
#  
#  Copyright 2016 notna <notna@apparat.org>
#  
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#  
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#  
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
#  MA 02110-1301, USA.
#  
#  

__all__ = ["PengWindow"]

import math

import pyglet
from pyglet.gl import *

from . import config


class PengWindow(pyglet.window.Window):
    """
    Main window class for peng3d and subclass of :py:class:`pyglet.window.Window()`\ .
    
    This class should not be instantiated directly, use the :py:meth:`Peng.createWindow()` method.
    """
    
    def __init__(self,peng,*args,**kwargs):
        super(PengWindow,self).__init__(*args,**kwargs)
        self.cameras = {}
        self.cam = None
        self.exclusive = False
        self.menus = {}
        self.activeMenu = None
        self.started = False
        self.cfg = config.Config({},defaults=peng.cfg)
        self._setup = False
    def setup(self):
        """
        Sets up the OpenGL state.
        
        This method should be called once after the config has been created and before the main loop is started.
        You should not need to manually call this method, as it is automatically called by :py:meth:`cleanConfig()`\ .
        
        Repeatedly calling this method has no effects.
        """
        if self._setup:
            return
        self._setup = True
        
        # Ensures that default values are supplied
        #self.cleanConfig()
        
        # Sets up basic OpenGL state
        glClearColor(*self.cfg["graphics.clearColor"])
        
        glEnable(GL_DEPTH_TEST)
        glEnable(GL_BLEND)
        
        glShadeModel(GL_SMOOTH)
        
        if self.cfg["graphics.fogSettings"]["enable"]:
            self.setupFog()
        if self.cfg["graphics.lightSettings"]["enable"]:
            self.setupLight()
    
    def run(self):
        """
        Runs the application in the current thread.
        
        This method should not be called directly, especially when using multiple windows, use :py:meth:`Peng.run()` instead.
        
        Note that this method is blocking as rendering needs to happen in the main thread.
        It is thus recommendable to run your game logic in another thread that should be started before calling this method.
        """
        self.setup()
        pyglet.app.run() # This currently just calls the basic pyglet main loop, maybe implement custom main loop for more control
    """def cleanConfig(self):
        ########## Reset quotation marks if uncommenting
        Sets default values for various config values.
        
        .. todo::
           
           Use an defaultdict or similiar instead.
        ########## End quotation marks
        # Various default values are set in this method, this should really be replaced with something more easy to use and robust
        # A possible replacement could be a defaultdict or similiar
        # Update: now replaced by :py:class:`Config()`\ .
        
        # OpenGL configs
        self.cfg["graphics.clearColor"] = self.cfg.get("graphics.clearColor",(0.,0.,0.,1.))
        self.cfg["graphics.wireframe"] = self.cfg.get("graphics.wireframe",False)
        self.cfg["graphics.fieldofview"] = self.cfg.get("graphics.fieldofview",65.0)
        self.cfg["graphics.nearclip"] = self.cfg.get("graphics.nearclip",0.1)
        self.cfg["graphics.farclip"] = self.cfg.get("graphics.farclip",10000) # It's over 9000!
        
        # OpenGL - Fog
        self.cfg["graphics.fogSettings"] = self.cfg.get("graphics.fogSettings",CFG_FOG_DEFAULT)
        self.cfg["graphics.fogSettings"]["enable"] = self.cfg["graphics.fogSettings"].get("enable",False)
        if self.cfg["graphics.fogSettings"]["enable"]:
            pass
        
        # OpenGL - Light
        self.cfg["graphics.lightSettings"] = self.cfg.get("graphics.lightSettings",CFG_LIGHT_DEFAULT)
        self.cfg["graphics.lightSettings"]["enable"] = self.cfg["graphics.lightSettings"].get("enable",False)
        if self.cfg["graphics.lightSettings"]["enable"]:
            pass
        
        # Other configs
        
        return
    """
    # Various methods
    def changeMenu(self,menu):
        """
        Changes to the given menu.
        
        ``menu`` must be a valid menu name that is currently known.
        
        This method will also pop any old handlers and push new handlers from the menu object.
        """
        if menu not in self.menus:
            raise ValueError("Menu %s does not exist!"%menu)
        elif menu == self.activeMenu:
            return # Ignore double menu activation to prevent bugs in menu initializer
        old = self.activeMenu
        self.activeMenu = menu
        if old is not None:
            self.menus[old].on_exit()
            self.pop_handlers()
        self.menu.on_enter(old)
        self.push_handlers(self.menu)
    def addMenu(self,menu):
        """
        Adds a menu to the list of menus.
        
        If there is no menu selected currently, this menu will automatically be made active.
        """
        self.menus[menu.name]=menu
        if self.activeMenu is None:
            self.changeMenu(menu.name)
    
    # Event handlers
    def on_draw(self):
        # Not documented
        # This just draws the appropriate menu
        self.clear()
        self.menu.draw()
    
    def on_key_press(self,symbol,modifiers):
        return True
    
    # Properties/Proxies for various things
    
    # Proxy for self.menus[self.activeMenu]
    @property
    def menu(self):
        """
        Property for accessing the currently active menu.
        
        Always equals ``self.menus[self.activeMenu]``\ .
        
        This property is read-only.
        """
        return self.menus[self.activeMenu]
    
    # Proxy for self.cam.rotation
    @property
    def rotation(self):
        """
        Property for accessing the current rotation of the active camera.
        
        This property can also be written to.
        """
        return self.cam.rotation
    @rotation.setter
    def rotation(self,value):
        self.cam.rotation = value
    
    # Proxy for self.cam.position
    @property
    def position(self):
        """
        Property for accessing the current position of the active camera.
        
        This property can also be written to.
        """
        return self.cam.position
    @position.setter
    def position(self,value):
        self.cam.position = value
    
    # Utility methods
    
    def set2d(self):
        """
        Configures OpenGL to draw in 2D.
        
        Note that wireframe mode is always disabled in 2D-Mode, but can be re-enabled by calling ``glPolygonMode(GL_FRONT_AND_BACK, GL_LINE)``\ .
        """
        # Light
        
        glDisable(GL_LIGHTING)
        
        # To avoid accidental wireframe GUIs and fonts
        glPolygonMode( GL_FRONT_AND_BACK, GL_FILL)
        
        width, height = self.get_size()
        glDisable(GL_DEPTH_TEST)
        glViewport(0, 0, width, height)
        glMatrixMode(GL_PROJECTION)
        glLoadIdentity()
        glOrtho(0, width, 0, height, -1, 1)
        glMatrixMode(GL_MODELVIEW)
        glLoadIdentity()
    def set3d(self):
        """
        Configures OpenGL to draw in 3D.
        
        This method also applies the correct rotation and translation as set in the current camera.
        It is discouraged to use :py:func:`glTranslatef()` or :py:func:`glRotatef()` directly as this may cause visual glitches.
        
        If you need to configure any of the standard parameters, see the docs about :doc:`/configoption`\ .
        
        The :confval:`graphics.wireframe` config value can be used to enable a wireframe mode, useful for debugging visual glitches.
        """
        
        # Light
        
        #glEnable(GL_LIGHTING)
        
        if self.cfg["graphics.wireframe"]:
            glPolygonMode(GL_FRONT_AND_BACK, GL_LINE)
        
        width, height = self.get_size()
        glEnable(GL_DEPTH_TEST)
        glViewport(0, 0, width, height)
        glMatrixMode(GL_PROJECTION)
        glLoadIdentity()
        gluPerspective(self.cfg["graphics.fieldofview"], width / float(height), self.cfg["graphics.nearclip"], self.cfg["graphics.farclip"]) # default 60
        glMatrixMode(GL_MODELVIEW)
        glLoadIdentity()
        x, y = self.rotation
        glRotatef(x, 0, 1, 0)
        glRotatef(-y, math.cos(math.radians(x)), 0, math.sin(math.radians(x)))
        x, y, z = self.position
        glTranslatef(-x, -y, -z)
